"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var swagger_jsdoc_1 = __importDefault(require("swagger-jsdoc"));
var lodash_1 = require("lodash");
exports.blueprintActions = ['findone', 'find', 'create', 'update', 'destroy', 'populate', 'add', 'remove', 'replace'];
exports.attributeValidations = [
    'isAfter',
    'isBefore',
    'isBoolean',
    'isCreditCard',
    'isEmail',
    'isHexColor',
    'isIn',
    'isInteger',
    'isIP',
    'isNotEmptyString',
    'isNotIn',
    'isNumber',
    'isString',
    'isURL',
    'isUUID',
    'max',
    'min',
    'maxLength',
    'minLength',
    'regex',
    'custom',
];
exports.loadSwaggerDocComments = function (filePath) {
    return new Promise(function (resolve, reject) {
        try {
            var opts = {
                definition: {
                    openapi: '3.0.0',
                    info: { title: 'dummy', version: '0.0.0' },
                },
                apis: [filePath],
            };
            var specification = swagger_jsdoc_1.default(opts);
            resolve(specification);
        }
        catch (err) {
            reject(err);
        }
    });
};
exports.getUniqueTagsFromPath = function (paths) {
    var referencedTags = new Set();
    for (var path in paths) {
        var pathDefinition = paths[path];
        for (var verb in pathDefinition) {
            var verbDefinition = pathDefinition[verb];
            if (verbDefinition.tags) {
                verbDefinition.tags.forEach(function (tag) { return referencedTags.add(tag); });
            }
        }
    }
    return referencedTags;
};
exports.resolveRef = function (specification, obj) {
    var path = obj.$ref;
    if (typeof (path) === 'string' && path.startsWith('#/')) {
        var pathElements = path.substring(2).split('/');
        return lodash_1.get(specification, pathElements);
    }
    return obj;
};
/**
 * Provides limited dereferencing, or unrolling, of schemas.
 *
 * Background: The generator `generateSchemas()` produces two variants:
 * 1. The `without-required-constraint` variant containing the properties but without
 *    the specifying required fields (used for update blueprint).
 * 2. A primary variant containing an `allOf` union of the variant above
 *    and the `required` constraint (used for the create blueprint).
 *
 * This method handles this simple case, resolving references and unrolling
 * into a simple cloned schema with directly contained properties.
 *
 * Otherwise, schema returned as clone but unmodified.
 *
 * @param specification
 * @param schema
 */
exports.unrollSchema = function (specification, schema) {
    var ret = lodash_1.cloneDeep(exports.resolveRef(specification, schema));
    if (ret.allOf) {
        var allOf = ret.allOf;
        delete ret.allOf;
        allOf.map(function (s) { return lodash_1.defaultsDeep(ret, exports.resolveRef(specification, s)); });
    }
    return ret;
};
